from datetime import datetime
from dateutil.parser import parse as dateParser

from .base import ObjectListModel, BaseModel

from .transports import Transport

class ShippingLabel(BaseModel):

    def __init__(self,
        transport=None,
        labelBytes=None
    ):

        self.transport = transport if transport else Transport()
        self.labelBytes = labelBytes

        self.hasError = False
        self.error = None
    
    def parse(self, headers, content):
        if 'X-Track-And-Trace-Code' in headers:
            self.transport.trackAndTrace = headers['X-Track-And-Trace-Code']
        
        if 'X-Transporter-Code' in headers:
            self.transport.transporterCode = headers['X-Transporter-Code']
        
        self.labelBytes = content

        return self

class DeliveryOptionList(ObjectListModel):

    def __init__(self):
        super(DeliveryOptionList, self).__init__(list=[])

        self.hasError = False
        self.error = None

    @property
    def options(self):
        return self.list

    def parse(self, json):
        if 'deliveryOptions' in json:
            for deliveryOption in json['deliveryOptions']:
                deliveryOptionObj = DeliveryOption().parse(deliveryOption)
                self.addToList(deliveryOptionObj)
        
        return self

class DeliveryOption(BaseModel):

    LABELTYPE_PARCEL = "PARCEL"
    LABELTYPE_MAILBOX = "MAILBOX"
    LABELTYPE_MAILBOX_LIGHT = "MAILBOX_LIGHT"

    def __init__(self,
        shippingLabelOfferId=None,
        validUntilDate=None,
        transporterCode=None,
        labelType=None,
        labelPrice=None,
        packageRestrictions=None,
        handoverDetails=None,
    ):

        self.shippingLabelOfferId = shippingLabelOfferId
        self.validUntilDate = validUntilDate
        self.transporterCode = transporterCode
        self.labelType = labelType
        self.labelPrice = labelPrice if labelPrice else LabelPrice()
        self.packageRestrictions = packageRestrictions if packageRestrictions else PackageRestrictions()
        self.handoverDetails = handoverDetails if handoverDetails else HandoverDetails()

        self.hasError = False
        self.error = None
    
    def parse(self, json):
        super(DeliveryOption, self).parse(json)

        if 'labelPrice' in json:
            self.labelPrice = LabelPrice().parse(json['labelPrice'])

        if 'packageRestrictions' in json:
            self.packageRestrictions = PackageRestrictions().parse(json['packageRestrictions'])
        
        if 'handoverDetails' in json:
            self.handoverDetails = HandoverDetails().parse(json['handoverDetails'])
        
        if self.validUntilDate: self.validUntilDate = datetime.strptime(self.validUntilDate, '%Y-%m-%d')

        return self


class LabelPrice(BaseModel):

    def __init__(self,
        totalPrice=None
    ):

        self.totalPrice = totalPrice

        self.hasError = False
        self.error = None


class PackageRestrictions(BaseModel):

    def __init__(self,
        maxWeight=None,
        maxDimensions=None
    ):
        
        self.maxWeight = maxWeight
        self.maxDimensions = maxDimensions

        self.hasError = False
        self.error = None

class HandoverDetails(BaseModel):

    COLLECTIONMETHOD_DROP_OFF = "DROP_OFF"
    COLLECTIONMETHOD_PICK_UP = "PICK_UP"

    def __init__(self,
        meetsCustomerExpectation=None,
        latestHandoverDateTime=None,
        collectionMethod=None
    ):

        self.meetsCustomerExpectation = meetsCustomerExpectation
        self.latestHandoverDateTime = latestHandoverDateTime
        self.collectionMethod = collectionMethod

        self.hasError = False
        self.error = None
    
    def parse(self, json):
        super(HandoverDetails, self).parse(json)
        if self.latestHandoverDateTime: self.latestHandoverDateTime = dateParser(self.latestHandoverDateTime)
        return self


from .base import ObjectListModel, BaseModel

class Error(ObjectListModel):

    def __init__(self,
        type=None,
        title=None,
        status=None,
        detail=None,
        host=None,
        instance=None,
    ):

        super(Error, self).__init__(list=[])

        self.type = type
        self.title = title
        self.status = status
        self.detail = detail
        self.host = host
        self.instance = instance
    
    @property
    def violations(self):
        return self.list
    
    def parse(self, json):
        
        super(Error, self).parse(json)

        if 'violations' in json:
            for violation in json['violations']:
                violation = Violation().parse(violation)
                self.addToList(violation)

        return self

class Violation(BaseModel):

    def __init__(self,
        name=None,
        reason=None
    ):
    
        self.name = name
        self.reason = reason


class Transporters:
    BRIEFPOST = "BRIEFPOST"
    UPS = "UPS"
    POSTNL = "TNT"
    POSTNL_EXTRA = "TNT-EXTRA"
    POSTNL_BRIEFPOST = "TNT_BRIEF"
    TNT_EXPRESS = "TNT-EXPRESS"
    DYNALOGIC = "DYL"
    DPD_NL = "DPD-NL"
    DPD_BE = "DPD-BE"
    BPOST_BE = "BPOST_BE"
    BPOST_BRIEFPOST = "BPOST_BRIEF"
    DHLFORYOU = "DHLFORYOU"
    GLS = "GLS"
    FEDEX_NL = "FEDEX_NL"
    FEDEX_BE = "FEDEX_BE"
    OTHER = "OTHER"
    DHL = "DHL"
    DHL_DE = "DHL_DE"
    DHL_GLOBAL_MAIL = "DHL-GLOBAL-MAIL"
    TRANSPORTSERVICE_NL = "TSN"
    FIEGE = "FIEGE"
    TRANSMISSION = "TRANSMISSION"
    PARCEL_NL = "PARCEL-NL"
    LOGOIX = "LOGOIX"
    PACKS = "PACKS"
    COURIER = "COURIER"
    NEDWRK = "RJP"


from .base import APIEndpoint

from bol.models.shippinglabels import DeliveryOptionList, DeliveryOption, ShippingLabel
from bol.models.processes import ProcessStatus
from bol.models.transports import Transport

class ShippingLabelMethods(APIEndpoint):

    def __init__(self, api):
        super(ShippingLabelMethods, self).__init__(api, "shipping-labels")

    def listOptions(self, order):

        if self.api.demo:
            data = { 'orderItems' : [{'orderItemId' : '2095052647'}]}
        else:
            data = { 'orderItems' : []}
            for item in order.orderItems:
                data['orderItems'].append({'orderItemId' : item.orderItemId })

        url = '{endpoint}/delivery-options'.format(endpoint=self.endpoint)

        status, headers, respJson = self.api.post(url, data)
        if status == 400: return DeliveryOptionList().parseError(respJson)

        return DeliveryOptionList().parse(respJson)

    def get(self, id):

        if self.api.demo: id = 'c628ba4f-f31a-4fac-a6a0-062326d0dbbd'

        url = '{endpoint}/{id}'.format(endpoint=self.endpoint, id=id)
        data = None

        labelHeaders = {
            'Accept' : 'application/vnd.retailer.v5+pdf',
            'Content-Type' : '',
        }

        status, headers, respContent = self.api.get(url, data, labelHeaders)
        if status == 400: return ShippingLabel().parseError(respContent)

        return ShippingLabel().parse(headers, respContent)
    
    def create(self, order, shippingLabelOfferId):

        if self.api.demo:
            data = { 'orderItems' : [{'orderItemId' : '2095052647'}], 'shippingLabelOfferId' : "8f956bfc-fabe-45b4-b0e1-1b52a0896b74"}
        else:
            data = { 'orderItems' : [], 'shippingLabelOfferId' : shippingLabelOfferId}

            for item in order.orderItems:
                data['orderItems'].append({'orderItemId' : item.orderItemId })

        url = self.endpoint

        status, headers, respJson = self.api.post(url, data)
        if status == 400: return ProcessStatus().parseError(respJson)
        
        return ProcessStatus().parse(respJson)


from .base import APIEndpoint

from bol.models.processes import ProcessStatus

class ProcessMethods(APIEndpoint):

    def __init__(self, api):
        super(ProcessMethods, self).__init__(api, "process-status")

    def get(self, id):

        if self.api.demo: id = 'c628ba4f-f31a-4fac-a6a0-062326d0dbbd'

        url = '{endpoint}/{id}'.format(endpoint=self.endpoint, id=id)
        data = None

        status, headers, respContent = self.api.get(url, data)
        if status == 400: return ProcessStatus().parseError(respContent)

        return ProcessStatus().parse(respContent)


from setuptools import setup

# read the contents of README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
  name = 'python-bol-retailer-api',         
  packages=['bol', 'bol.models', 'bol.constants', 'bol.cache', 'bol.endpoints'],
  version = '0.5.2',
  license='GPL-3.0-or-later',
  description = 'Wrapper for the bol.com Retailer API (v5)',
  long_description=long_description,
  long_description_content_type='text/markdown',
  author = 'Alexander Schillemans',
  author_email = 'alexander.schillemans@lhs.global',
  url = 'https://github.com/alexanderlhsglobal/python-bol-retailer-api',
  download_url = 'https://github.com/alexanderlhsglobal/python-bol-retailer-api/archive/refs/tags/v0.5.2.tar.gz',
  keywords = ['bol.com', 'api', 'bol', 'wrapper'],
  install_requires=[
          'requests',
          'python-dateutil',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Programming Language :: Python :: 3.6',
  ],
)